/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    \file FeatureVector.h

    $Author: Frank Moosmann $ ( < moosmann@mrt.uka.de>)
    $Date: 2005/11/07 $
*///////////////////////////////////////////////////////////////////////////

 
#ifndef FEATURE_VECTOR_H
#define FEATURE_VECTOR_H

#include <vector>
#include <list>
#include <ostream>
#include <istream>
#include "ClassLabel.h"

typedef float FeatureData;
#define FEATURE_NULL 0.0;

class FeatureVector {
  public:
    FeatureVector();
    ~FeatureVector();
  
    void addElement(FeatureData elem);
    FeatureData getElement(int index); //!< index in range 0..getSize()-1
    unsigned int getSize();
    void setElement(int index, FeatureData elem); //!< index in range 0..getSize()-1
    
    std::string imageName;
    unsigned int imageWidth;    //!< in pixel
    unsigned int imageHeight;   //!< in pixel
    unsigned int windowPosX;    //!< in pixel
    unsigned int windowPosY;    //!< in pixel
    unsigned int windowSize;    //!< scale in pixel
    double windowToObjectRatio; //!< with respect to area
    ClassLabel classLabel;
    
    static FeatureData compEmpMean (std::list<FeatureVector*> &featVecList, int attrNo);
    static FeatureData compStdDeviation (std::list<FeatureVector*> &featVecList, int attrNo, FeatureData mean);
    
    static void writeIntoStream(std::list<FeatureVector*> &featureList, std::ostream &data, std::ostream &info, int maxcount = 0);
    static int  readFromStream(std::istream &data, std::istream &info, std::list<FeatureVector*> &featureList, int noToRead = 0);
    
  private:
    std::vector<FeatureData> data;
    unsigned int size;
};



#endif /*FEATURE_VECTOR_H*/
