/***************************************************************************
                                MaskImage.cpp
                             ------------------
    begin                : 28.11.2005
    copyright            : (C) 2005 by Frank Moosmann and INRIA
    email                : moosmann@mrt.uka.de
 ***************************************************************************/

#include "MaskImage.h"
#include <iostream>
#include <fstream>

using namespace Magick;
using namespace std;

 
/***************************************************************************
                         Constructors / Destructors
 ***************************************************************************/
MaskImage::MaskImage(string filename, unsigned int im_width, unsigned int im_height)
{
  imWidth = im_width;
  imHeight = im_height;
  sourceImage.read(filename.c_str());
  sourceImage.scale(Geometry(imWidth,imHeight));      // Resize mask to size of image
  createMaskIntegral();
}


MaskImage::~MaskImage()
{
  deleteMaskIntegral();
}


/***************************************************************************
                         public Methods
 ***************************************************************************/
double MaskImage::getObjBgrRatio(unsigned int width, unsigned int height, unsigned int offset_x, unsigned int offset_y)
{
  if (((width+offset_x) > imWidth) || ((height+offset_y) > imHeight))
    return 0.0;
  unsigned int noTotPix = width*height;
  unsigned int noObjPix = maskIntegral[offset_x+width][offset_y+height]
                         +maskIntegral[offset_x][offset_y]
                         -maskIntegral[offset_x+width][offset_y]
                         -maskIntegral[offset_x][offset_y+height];
  return (double)noObjPix / (double)noTotPix;
}


void MaskImage::printIntoFile(string filename)
{
  ofstream outFile;
  outFile.open(filename.c_str(), ios::out);
  for (unsigned int row=1; row<imHeight; row++) {
    for (unsigned int col=1; col<imWidth; col++) {
      if (!(ColorMono(sourceImage.pixelColor(col,row)).mono()))
        outFile << 1 << " ";
      else
        outFile << 0 << " ";
    }
    outFile << endl;
  }
  outFile << endl;
  for (unsigned int row=1; row<=imHeight; row++) {
    for (unsigned int col=1; col<=imWidth; col++) {
      outFile << maskIntegral[col][row] << " ";
    }
    outFile << endl;
  }
  outFile.close();
}


/***************************************************************************
                         private Methods
 ***************************************************************************/
void MaskImage::createMaskIntegral()
{
  // allocate Memory
  maskIntegral             = new unsigned int* [imWidth+1]; // maskIntegral[imWidth+1][imHeight+1]
  unsigned int **cumRowSum = new unsigned int* [imWidth+1]; // cumRowSum[imWidth+1][imHeight+1]
  for(unsigned int i=0;i<=imWidth;i++) {
    maskIntegral[i] = new unsigned int[imHeight+1];
    cumRowSum[i]    = new unsigned int[imHeight+1];
  }
  
  // do processing for first row
  for (unsigned int col=0; col<=imWidth; col++) {
    cumRowSum[col][0] = 0;
    maskIntegral[col][0] = 0;
  }
  // do processing for rest of image
  for (unsigned int row=1; row<=imHeight; row++) {
    // do processing for first column
    cumRowSum[0][row] = 0;
    maskIntegral[0][row] = 0;
    // do processing for rest of image
    for (unsigned int col=1; col<=imWidth; col++) {
      unsigned int pixelVal = 0;
      if (!(ColorMono(sourceImage.pixelColor(col-1,row-1)).mono()))
        pixelVal = 1;
      else
        pixelVal = 0;
      cumRowSum[col][row] = cumRowSum[col-1][row] + pixelVal;
      maskIntegral[col][row] = maskIntegral[col][row-1] + cumRowSum[col][row];
    }
  }
  
  for(unsigned int i=0;i<=imWidth;i++) 
    delete [] cumRowSum[i];
  delete [] cumRowSum;
}


void MaskImage::deleteMaskIntegral()
{
  for(unsigned int i=0;i<=imWidth;i++)
    delete [] maskIntegral[i];
  delete [] maskIntegral;
}


